create or replace package HPM is

  -- Author  : POMATTI
  -- Created : 20/05/2007 21.01.34
  -- Purpose :

  -- Public type declarations
  type ref_cursor is ref cursor;
  type date_array is table of date index by binary_integer;

  -- Public constant declarations

  -- Public variable declarations

  -- Public function and procedure declarations
  function CODIFICA_TENDENZA(val in number) return number;
  function CUMGIORNALIERA_R(ndas in varchar2, daterif in date) return number;
  function CUMGIORNALIERA_S(cstaz in varchar2, csens in varchar2, daterif in date) return number;
  function CUMULATA_R(ndas in varchar2, datefrom in date, dateto in date) return number;
  function CUMULATA_S(cstaz in number, csens in number, datefrom in date, dateto in date) return number;
  function DATAPIENA(cstaz in varchar2, csens in varchar2, datefrom in date, dateto in date) return date;
  function DATAPIU9ORE(daterif in date) return date;
  function FORTE_DATA(cstaz number, csens number) return date;
  function FORTE_VALORE(cstaz number, csens number) return number;
  function SR(cstaz number, csens number, val number, tsoglia integer) return integer;
  function FORTE_SR_SENS(cstaz number, csens number, tsoglia integer) return integer;
  function FORTE_SRU_SENS(val number, great number, sogliaOrd number, sogliaMod number, sogliaHig number) return integer;
  function GET_HPMHS_PARAMETER(sensorId in number, geoField in string, geoId in string) return ref_cursor;
  function GET_HPMHS_PARAMETERS_LIST(geoField in string, geoId in string) return ref_cursor;
  function GETSOLOSTAZIONE(cstaz in varchar2) return char;
  function GETSTAZSENS(ndas in varchar2) return char;
  function TENDENZA_R(ndas in varchar2, datarif in date) return integer;
  function TENDENZA_R_CUM(ndas in varchar2, datarif in date) return number;
  function VALELEM_R(ndas in varchar2, datarif in date) return number;
  function VALELEM_S(cstaz in number, csens in number, datarif in date) return number;
  function VALPREC_R(ndas in varchar2, datarif in date) return number;
  function VALPREC_S(cstaz number, csens number, daterif in date) return number;
  function GET_PORTATA(cstaz number, h number) return number;
end HPM;


 
/
create or replace package body HPM is

  -- Private type declarations

  -- Private constant declarations

  -- Private variable declarations

  -- Function and procedure implementations

  function CODIFICA_TENDENZA(val in number) return number is
    -- PL/SQL Specification
    cod integer;

    -- PL/SQL Block
    begin
      cod := -1;
      if val > 0 then
        cod := 2;
      elsif val = 0 then
        cod := 1;
      elsif val < 0 then
        cod := 0;
      end if;

      return cod;
  end;

  function CUMGIORNALIERA_R(ndas in varchar2, daterif in date) return number is
    -- PL/SQL Specification
    cum number;

    -- PL/SQL Block
    begin
      cum := CUMULATA_R(ndas, daterif - 1, daterif);
      return cum;
  end;

  function CUMGIORNALIERA_S(cstaz in varchar2, csens in varchar2, daterif in date) return number is
    -- PL/SQL Specification
    cum number;

    -- PL/SQL Block
    begin
      cum := CUMULATA_S(cstaz, csens, daterif - 1, daterif);
      return cum;
  end;

  function CUMULATA_R(ndas in varchar2, datefrom in date, dateto in date) return number is
    -- PL/SQL Specification
    cum number;
    val_cur number;
    val_prec number;
      
    cursor data_cur(ndas in varchar2, datefrom date, dateto date) is
      select VALORE from monitoraggio.MISURE
      where NUMERO_DAS = ndas and STATO = 1 and VALORE is not null
        and DATA_RILEVAMENTO between datefrom - 1/288 and dateto + 1/288
      order by DATA_RILEVAMENTO;

    -- PL/SQL Block
    begin
      cum := 0;
      val_prec := null;
      for cur in data_cur(ndas, datefrom, dateto) loop
        val_cur := cur.VALORE;
        if val_prec is null then
          val_prec := val_cur;
        end if;
        
        if val_cur >= val_prec then
          cum := cum + (val_cur - val_prec);
        end if;
        if val_cur < val_prec then
          if abs(val_cur - val_prec) > 0.5 then
            cum := cum + val_cur;
          end if;
        end if;
        val_prec := val_cur;

        exit when data_cur%notfound;
      end loop;

      return cum;
  end;

  function CUMULATA_S(cstaz in number, csens in number, datefrom in date, dateto in date) return number is
    -- PL/SQL Specification
    cum number;
    val_cur number;
    val_prec number;
      
    cursor data_cur(cstaz in number, csens in number, datefrom date, dateto date) is
      select VALORE from monitoraggio.PIOGGIA
      where COD_STAZIONE = cstaz and COD_SENSORE = csens
        and STATO = 1 and VALORE is not null
        and DATA_RILEVAMENTO between datefrom - 1/288 and dateto + 1/288
      order by DATA_RILEVAMENTO;

    -- PL/SQL Block
    begin
      cum := 0;
      val_prec := null;
      for cur in data_cur(cstaz, csens, datefrom, dateto) loop
        val_cur := cur.VALORE;
        if val_prec is null then
          val_prec := val_cur;
        end if;
        
        if val_cur >= val_prec then
          cum := cum + (val_cur - val_prec);
        end if;
        if val_cur < val_prec then
          if abs(val_cur - val_prec) > 0.5 then
            cum := cum + val_cur;
          end if;
        end if;
        val_prec := val_cur;

        exit when data_cur%notfound;
      end loop;

      return cum;
  end;

  function DATAPIENA(cstaz in varchar2, csens in varchar2, datefrom in date, dateto in date) return date is
    -- PL/SQL Specification
    daterif date;

    -- PL/SQL Block
    begin
      select max(DATA_RILEVAMENTO) into daterif from monitoraggio.LIVELLO_IDROMETRICO
      where VALORE in (
        select distinct max(VALORE) from monitoraggio.LIVELLO_IDROMETRICO
        where COD_STAZIONE = cstaz and COD_SENSORE = csens and
        DATA_RILEVAMENTO between datefrom - 1/288 and dateto + 1/288 and STATO = 1)
      and COD_STAZIONE = cstaz and COD_SENSORE = csens and
      DATA_RILEVAMENTO between datefrom - 1/288 and dateto + 1/288 and STATO = 1;

      return daterif;
  end;

  function DATAPIU9ORE(daterif in date) return date is
    -- PL/SQL Specification
    val  date;

    -- PL/SQL Block
    begin
      select daterif + 9/24 into val from dual;
      return val;
  END;

  function FORTE_DATA(cstaz number, csens number) return date is
    -- PL/SQL Specification
    cursor misure_cur(cstaz number, csens number) is
      select VAL_ALLARME valore, ORA_ULTIMO_DATOVAL ora
    	from monitoraggio.SINTESI_MISURE
    	where COD_STAZIONE = cstaz and COD_SENSORE = csens;
    misure_rec misure_cur%ROWTYPE;
    daterif date;

    -- PL/SQL Block
    begin
      for misure_rec in misure_cur(cstaz, csens) LOOP
          daterif := misure_rec.ora;
      end loop;

  	  return daterif;
  end;

  function FORTE_VALORE(cstaz number, csens number) return number is
    -- PL/SQL Specification
    cursor misure_cur(cstaz number, csens number) is
      select VAL_ALLARME
    	from monitoraggio.SINTESI_MISURE
    	where COD_STAZIONE = cstaz and COD_SENSORE = csens;
    misure_rec misure_cur%ROWTYPE;
    val	number;

    -- PL/SQL Block
    begin
    	val := null;
    	for misure_rec in misure_cur(cstaz, csens) loop
    		if misure_rec.VAL_ALLARME is not null then
    			val := misure_rec.VAL_ALLARME;
    			exit;
    		end if;
    		exit when misure_cur%NOTFOUND;
    	end loop;

    	--dbms_output.put_line('fine: '||to_char(valore));
    	return val;

    exception
      when NO_DATA_FOUND then
      return -999;
  end;

  function SR(cstaz number, csens number, val number, tsoglia integer) return integer is
    -- PL/SQL Specification
    cursor soglie_staz_cur(cod_st number, cod_se number, tipo_s integer) is
    		select SOGLIA_ORDINARIA, SOGLIA_MODERATA, SOGLIA_ELEVATA, SOGLIA_ESTREMA
    		from SOGLIE_ALLARMI
    		where COD_STAZIONE = cod_st and COD_SENSORE = cod_se and
    			COD_TIPO_SOGLIA = tipo_s;
    cursor soglie_areo_cur(cod_st number, cod_se number, tipo_s integer) is
        select b.SOGLIA_ORDINARIA, b.SOGLIA_MODERATA, b.SOGLIA_ELEVATA, b.SOGLIA_ESTREMA
        from
          (select COD_SENSORE, COD_STAZIONE, COD_AREAOMO from monitoraggio.SINTESI_RETE) a,
          (select COD_AREEOMO, SOGLIA_ORDINARIA, SOGLIA_MODERATA, SOGLIA_ELEVATA, SOGLIA_ESTREMA from SOGLIE_ALLARMI_OMO where COD_TIPO_SOGLIA = tipo_s) b
        where a.COD_AREAOMO = b.COD_AREEOMO (+)
          and a.COD_STAZIONE = cod_st and a.COD_SENSORE  = cod_se;
    soglie_staz_rec soglie_staz_cur%ROWTYPE;
    soglie_areo_rec soglie_areo_cur%ROWTYPE;
    stato integer;

    -- PL/SQL Block
    begin
      stato := 1;
      if val is null then
         stato := 0;
    	else
    		if val = -999 then
    			stato := 0;
    		else
          for soglie_staz_rec in soglie_staz_cur(cstaz, csens, tsoglia) loop
            stato := 2; -- Normale
            if (val >= soglie_staz_rec.SOGLIA_ORDINARIA) then
              stato := 3; -- ordinaria criticit?
            end if;
            if (val >= soglie_staz_rec.SOGLIA_MODERATA) then
              stato := 4; -- moderata criticit?
            end if;
            if (val >= soglie_staz_rec.SOGLIA_ELEVATA) then
              stato := 5; -- elevata criticit?
            end if;
            if (val >= soglie_staz_rec.SOGLIA_ESTREMA) then
              stato := 6; -- estrema criticit?
            end if;
            exit;
          end loop;

          if stato = 1 and csens = 301 then
            for soglie_areo_rec in soglie_areo_cur(cstaz, csens, tsoglia) loop
              stato := 2; -- Normale
              if (soglie_areo_rec.SOGLIA_ORDINARIA is null
                 and soglie_areo_rec.SOGLIA_MODERATA is null
                 and soglie_areo_rec.SOGLIA_ELEVATA is null
                 and soglie_areo_rec.SOGLIA_ESTREMA is null) then
                stato := 1; -- nessuna soglia
              end if;
              if (val >= soglie_areo_rec.SOGLIA_ORDINARIA) then
                stato := 3; -- ordinaria criticit?
              end if;
              if (val >= soglie_areo_rec.SOGLIA_MODERATA) then
                stato := 4; -- moderata criticit?
              end if;
              if (val >= soglie_areo_rec.SOGLIA_ELEVATA) then
                stato := 5; -- elevata criticit?
              end if;
              if (val >= soglie_areo_rec.SOGLIA_ESTREMA) then
                stato := 6; -- estrema criticit?
              end if;
              exit;
            end loop;
          end if;
    	  end if;
    	end if;

    	return stato;
  end;

  function FORTE_SR_SENS(cstaz number, csens number, tsoglia integer) return integer is
    -- PL/SQL Specification
    stato	integer;
    val number;

    -- PL/SQL Block
    begin
    	val := FORTE_VALORE(cstaz, csens);
    	stato := SR(cstaz, csens, val, tsoglia);

    	return stato;
  end;

  function FORTE_SRU_SENS(val number, great number, sogliaOrd number, sogliaMod number, sogliaHig number) return integer is
    -- PL/SQL Specification
    stato integer;

    -- PL/SQL Block
    begin
    	stato := 0;
    	if val is not null then
    		if val = -999  then
    			stato := 0;
    		elsif sogliaOrd is null and sogliaMod is null and sogliaHig is null then
    			stato := 1;
        else
          stato := 2; -- Normale
          if great = 1 then
            if (val >= sogliaOrd) then
              stato := 3; -- ordinaria criticit?
            end if;
            if (val >= sogliaMod) then
              stato := 4; -- moderata criticit?
            end if;
            if (val >= sogliaHig) then
              stato := 5; -- elevata criticit?
            end if;
          else
            if (val <= sogliaOrd) then
              stato := 3; -- ordinaria criticit?
            end if;
            if (val <= sogliaMod) then
              stato := 4; -- moderata criticit?
            end if;
            if (val <= sogliaHig) then
              stato := 5; -- elevata criticit?
            end if;
          end if;
    		end if;
    	end if;

    	return stato;
  end;

  function GET_HPMHS_PARAMETER(sensorId in number, geoField in string, geoId in string) return ref_cursor is
    -- PL/SQL Specification
    date_max DATE;
    res_cur ref_cursor;

    -- PL/SQL Block
    begin
      execute immediate 'select max(DATAF_RT) from monitoraggio.SINTESI_RETE a, monitoraggio.SINTESI_MISURE b ' ||
              'where a.NUMERO_DAS = b.NUMERO_DAS and a.' || geoField || ' = :geoId' into date_max using geoId;

      if date_max is null then
         open res_cur for 'select distinct a.COD_SENSORE, b.PARAMETRO, min(c.DATAI_ST), max(c.DATAF_ST), d.SIMBOLO, b.TABELLA_DATI ' ||
               'from monitoraggio.SINTESI_RETE a, monitoraggio.SENSORI b, monitoraggio.SINTESI_DATISTORICI c, monitoraggio.UNITA_MISURA d ' ||
               'where a.NUMERO_DAS = c.NUMERO_DAS and b.COD_UNITA_MISURA_VIS = d.COD_UNITA_MISURA and a.COD_SENSORE = b.COD_SENSORE and a.' || geoField || ' = :geoId and b.COD_SENSORE = :sensorId group by a.COD_SENSORE, b.PARAMETRO, d.SIMBOLO, b.TABELLA_DATI' using geoId, sensorId;
      else
         open res_cur for 'select distinct a.COD_SENSORE, b.PARAMETRO, min(c.DATAI_ST), max(e.DATAF_RT), d.SIMBOLO, b.TABELLA_DATI ' ||
               'from monitoraggio.SINTESI_RETE a, monitoraggio.SENSORI b, monitoraggio.SINTESI_DATISTORICI c, monitoraggio.UNITA_MISURA d, monitoraggio.SINTESI_MISURE e ' ||
               'where a.NUMERO_DAS = c.NUMERO_DAS and a.NUMERO_DAS = e.NUMERO_DAS and b.COD_UNITA_MISURA_VIS = d.COD_UNITA_MISURA and a.COD_SENSORE = b.COD_SENSORE and a.' || geoField || ' = :geoId and b.COD_SENSORE = :sensorId group by a.COD_SENSORE, b.PARAMETRO, d.SIMBOLO, b.TABELLA_DATI' using geoId, sensorId;
      end if;

    return res_cur;
  end;

  function GET_HPMHS_PARAMETERS_LIST(geoField in string, geoId in string) return ref_cursor is
    -- PL/SQL Specification
    date_max DATE;
  	res_cur ref_cursor;

    -- PL/SQL Block
    begin
      execute immediate 'select max(DATAF_RT) from monitoraggio.SINTESI_RETE a, monitoraggio.SINTESI_MISURE b ' ||
              'where a.NUMERO_DAS = b.NUMERO_DAS and a.' || geoField || ' = :geoId' into date_max using geoId;

      if date_max is null then
         open res_cur for 'select distinct a.COD_SENSORE, b.PARAMETRO, min(c.DATAI_ST), max(c.DATAF_ST) ' ||
              'from monitoraggio.SINTESI_RETE a, monitoraggio.SENSORI b, monitoraggio.SINTESI_DATISTORICI c ' ||
              'where a.NUMERO_DAS = c.NUMERO_DAS and a.COD_SENSORE = b.COD_SENSORE and a.' || geoField || ' = :geoId group by a.COD_SENSORE, b.PARAMETRO order by b.PARAMETRO' using geoId;
      else
         open res_cur for 'select distinct a.COD_SENSORE, b.PARAMETRO, min(c.DATAI_ST), max(d.DATAF_RT) ' ||
              'from monitoraggio.SINTESI_RETE a, monitoraggio.SENSORI b, monitoraggio.SINTESI_DATISTORICI c, monitoraggio.SINTESI_MISURE d ' ||
              'where a.NUMERO_DAS = c.NUMERO_DAS and a.NUMERO_DAS = d.NUMERO_DAS and a.COD_SENSORE = b.COD_SENSORE and a.' || geoField || ' = :geoId group by a.COD_SENSORE, b.PARAMETRO order by b.PARAMETRO' using geoId;
      end if;

    return res_cur;
  end;

  function GETSOLOSTAZIONE(cstaz in varchar2) return char is
    -- PL/SQL Specification
    nome varchar2(50);

    -- PL/SQL Block
    begin
      select STAZIONE into nome
      from monitoraggio.STAZIONI
      where COD_STAZIONE = cstaz;

    return nome;
  end;

  function GETSTAZSENS(ndas in varchar2) return char is
    -- PL/SQL Specification
    nome varchar2(120);

    -- PL/SQL Block
    begin
      select COD_RETE || ' - ' || STAZIONE || ' - ' || PARAMETRO into nome
      from CONF_STAZIONI a, STAZIONI b, SENSORI c
      where a.NUMERO_DAS = ndas and
        a.COD_STAZIONE = b.COD_STAZIONE and
        a.COD_SENSORE = c.COD_SENSORE;

    return nome;
  end;

  function TENDENZA_R(ndas in varchar2, datarif in date) return integer is
    -- PL/SQL Specification
    val1 number;
    val2 number;
    dtlast date;

    -- PL/SQL Block
    begin
      val1 := VALPREC_R(ndas, datarif);

      select max(DATA_RILEVAMENTO) into dtlast from monitoraggio.MISURE
      where NUMERO_DAS = ndas and STATO = 1 and
        DATA_RILEVAMENTO between datarif - 3/24 and datarif;

      val2 := VALPREC_R(ndas, dtlast - 2/24);

      return CODIFICA_TENDENZA(val1 - val2);
  end;

  function TENDENZA_R_CUM(ndas in varchar2, datarif in date) return number is
    -- PL/SQL Specification
    val11 number;
    val12 number;
    ist1 number;
    ist2 number;
    dtlast date;

    -- PL/SQL Block
    begin
      val11 := VALPREC_R(ndas, datarif);

      select max(DATA_RILEVAMENTO) into dtlast from monitoraggio.MISURE
      where NUMERO_DAS = ndas and STATO = 1 and
        DATA_RILEVAMENTO between datarif - 3/24 and datarif;

      val12 := VALPREC_R(ndas, dtlast);
      ist1 := val11 - val12;

      select max(DATA_RILEVAMENTO) into dtlast from monitoraggio.MISURE
      where NUMERO_DAS = ndas and STATO = 1 and
        DATA_RILEVAMENTO between dtlast - 3/24 and dtlast;

      ist2 := val12 - VALPREC_R(ndas, dtlast);

      return CODIFICA_TENDENZA(ist1 - ist2);
  end;

  function VALELEM_R(ndas in varchar2, datarif in date) return number is
    -- PL/SQL Specification
    val1 number;
    val2 number;

    -- PL/SQL Block
    begin
      select VALORE into val1 from monitoraggio.MISURE
      where DATA_RILEVAMENTO in (
        select max(DATA_RILEVAMENTO) from monitoraggio.MISURE
        where	DATA_RILEVAMENTO between datarif - 1/288 and datarif + 1/288 and
        NUMERO_DAS = ndas and STATO = 1) and
      NUMERO_DAS = ndas and STATO = 1;

      select VALORE into val2 from monitoraggio.MISURE
      where DATA_RILEVAMENTO in (
        select max(DATA_RILEVAMENTO) from monitoraggio.MISURE
        where	DATA_RILEVAMENTO between datarif - 7/288 and datarif - 1/288 and
        NUMERO_DAS = ndas  and STATO = 1) and
      NUMERO_DAS = ndas and STATO = 1;

      val1 := val1 - val2;

      return val1;
  end;

  function VALELEM_S(cstaz in number, csens in number, datarif in date) return number is
    -- PL/SQL Specification
    val1 number;
    val2 number;

    -- PL/SQL Block
    begin
      select VALORE	into val1 from monitoraggio.PIOGGIA
      where DATA_RILEVAMENTO in (
        select max(DATA_RILEVAMENTO) from monitoraggio.PIOGGIA
        where	DATA_RILEVAMENTO between datarif - 1/288 and datarif + 1/288 and
        COD_STAZIONE = cstaz and COD_SENSORE = csens and STATO = 1) and
      COD_STAZIONE = cstaz and COD_SENSORE = csens and STATO = 1;

      select VALORE	into val2 from monitoraggio.PIOGGIA
      where DATA_RILEVAMENTO in (
        select max(DATA_RILEVAMENTO) from monitoraggio.PIOGGIA
        where	DATA_RILEVAMENTO between datarif - 7/288 and datarif - 1/288 and
        COD_STAZIONE = cstaz and COD_SENSORE = csens and STATO = 1) and
      COD_STAZIONE = cstaz and COD_SENSORE = csens and STATO = 1;

      val1 := val1 - val2;

      return val1;
  end;

  function VALPREC_R(ndas in varchar2, datarif in date) return number is
    -- PL/SQL Specification
    val number;

    -- PL/SQL Block
    begin
      select VALORE into val from monitoraggio.MISURE
      where DATA_RILEVAMENTO in (
        select max(DATA_RILEVAMENTO) from monitoraggio.MISURE
        where NUMERO_DAS = ndas and STATO = 1 and
        DATA_RILEVAMENTO between datarif - 3/24 and datarif - 1/288)
      and NUMERO_DAS = ndas and STATO = 1;

      return val;

    exception
    	when NO_DATA_FOUND then
    	return null;
  end;

  function VALPREC_S(cstaz number, csens number, daterif in date) return number is
    -- PL/SQL Specification
    val number;

    -- PL/SQL Block
    begin
      select VALORE into val from monitoraggio.PIOGGIA
      where DATA_RILEVAMENTO in (
        select max(DATA_RILEVAMENTO) from monitoraggio.PIOGGIA
        where COD_SENSORE = csens and COD_STAZIONE = cstaz and STATO = 1
        and DATA_RILEVAMENTO between daterif - 3/24 and daterif - 1/288)
      and COD_SENSORE = csens and COD_STAZIONE = cstaz and STATO = 1;

      return val;

    exception
    	when NO_DATA_FOUND then
    	return null;
  end;

  function GET_PORTATA(cstaz number, h number) return number is
    q number;
    a number;
    val number;
    f number;
    num number;

    begin
      q := null;
      if (h is null) then
        return null;
      end if;

      -- H_MIN<h<=H_MAX
      select count(*)
        into num
        from DATI_SCALE_PORTATA p
       where p.COD_STAZIONE = cstaz
         and p.H_MIN < h
         and h <= p.H_MAX;

      if num > 0 then
        -- a*(b+c*h+d*h^2+e*h^3)^f
        select p.a, (p.B + p.C * h + p.D * h * h + p.E * h * h * h), p.F
          into a, val, f
          from DATI_SCALE_PORTATA p
         where p.COD_STAZIONE = cstaz
           and p.H_MIN < h
           and h <= p.H_MAX;

        if val < 0 then
          q := a * power(val, round(f));
        else
          q := a * power(val, f);
        end if;
      end if;

      return q;
  end;
end HPM;
/
